;**************************** Librera "LCD_MENS.INC" 
;
;
;
; Librera de subrutinas para el manejo de mensajes a visualizar en un 

visualizador LCD.

    CBLOCK
    LCD_ApuntaCaracter                ; Indica la posicin del carcter a 

visualizar
                                    ; respecto del comienzo de todos los 

mensajes,
                                    ; (posicin de la etiqueta "Mensajes").
    LCD_ValorCaracter                ; Cdigo ASCII del carcter a
    ENDC                            ; visualizar.

; Los mensajes tienen que estar situados dentro de las 256 primeras 

posiciones de la
; memoria de programa, es decir, no pueden superar la direccin 0FFh.

; Subrutina "LCD_Mensaje" 

---------------------------------------------------------------
;
; Visualiza por pantalla el mensaje apuntado por el registro W.
;
; Los mensajes deben localizarse dentro de una zona encabezada por la 

etiqueta "Mensajes" y que
; tenga la siguiente estructura:
;
; Mensajes                            ; Etiqueta obligatoria!
;    addwf    PCL,F
; Mensaje0                            ; Posicin inicial del mensaje.
;    DT ".. ..", 0x00                ; Mensaje terminado en 0x00.
; Mensaje1
;    ...   
;    ...
; FinMensajes
;
; La llamada a esta subrutina se realizar siguiendo este ejemplo:
;
;    movlw    Mensaje0                ; Carga la posicin del mensaje.
;    call    LCD_Mensaje                ; Visualiza el mensaje.
;
LCD_Mensaje
    movwf    LCD_ApuntaCaracter        ; Posicin del primer carcter del 

mensaje.
    movlw    Mensajes                ; Halla la posicin relativa del primer 

carcter
    subwf    LCD_ApuntaCaracter,F    ; del mensaje respecto de etiqueta 

"Mensajes".
    decf    LCD_ApuntaCaracter,F    ; Compensa la posicin que ocupa "addwf 

PCL,F".
LCD_VisualizaOtroCaracter
    movf    LCD_ApuntaCaracter,W
    call    Mensajes                ; Obtiene el cdigo ASCII del carcter 

apuntado.
    movwf    LCD_ValorCaracter        ; Guarda el valor de carcter.
    movf    LCD_ValorCaracter,F        ; Lo nico que hace es posicionar flag 

Z. En caso
    btfsc    STATUS,Z                ; que sea "0x00", que es cdigo 

indicador final   
    goto    LCD_FinMensaje            ; de mensaje, sale fuera.
LCD_NoUltimoCaracter
    call    LCD_Caracter            ; Visualiza el carcter ASCII ledo.
    incf    LCD_ApuntaCaracter,F    ; Apunta a la posicin del siguiente 

carcter
    goto    LCD_VisualizaOtroCaracter    ; dentro del mensaje.
LCD_FinMensaje
    return                            ; Vuelve al programa principal.

; Subrutina "LCD_MensajeMovimiento" 

-----------------------------------------------------
;
; Visualiza un mensaje de mayor longitud que los 16 caracteres que pueden 

representarse
; en una lnea, por tanto se desplaza a travs de la pantalla.
;
; En el mensaje debe dejarse 16 espacios en blanco, al principio y al final 

para
; conseguir que el desplazamiento del mensaje sea lo ms legible posible.
;
    CBLOCK   
    LCD_CursorPosicion                ; Contabiliza la posicin del cursor 

dentro de la
    ENDC                            ; pantalla LCD

LCD_MensajeMovimiento
    movwf    LCD_ApuntaCaracter        ; Posicin del primer carcter del 

mensaje.
    movlw    Mensajes                ; Halla la posicin relativa del primer 

carcter
    subwf    LCD_ApuntaCaracter,F    ; del mensaje respecto de la etiqueta 

"Mensajes".
    decf    LCD_ApuntaCaracter,F    ; Compensa la posicin que ocupa "addwf 

PCL,F".
LCD_PrimeraPosicion
    clrf    LCD_CursorPosicion        ; El cursor en la posicin 0 de la 

lnea.
    call    LCD_Borra                ; Se sita en la primera posicin de la 

lnea 1 y
LCD_VisualizaCaracter                ; borra la pantalla.
    movlw    LCD_CaracteresPorLinea    ; Ha llegado a final de lnea?
    subwf    LCD_CursorPosicion,W
    btfss    STATUS,Z
    goto    LCD_NoEsFinalLinea
LCD_EsFinalLinea
    call    Retardo_200ms            ; Lo mantiene visualizado durante este 

tiempo.
    call    Retardo_200ms
    movlw    LCD_CaracteresPorLinea-1; Apunta a la posicin del segundo 

carcter visualizado
    subwf    LCD_ApuntaCaracter,F    ; en pantalla, que ser el primero en la 

siguiente
    goto    LCD_PrimeraPosicion         ; visualizacin de lnea, para 

producir el efecto
LCD_NoEsFinalLinea                    ; de desplazamiento hacia la izquierda.
    movf    LCD_ApuntaCaracter,W
    call    Mensajes                ; Obtiene el ASCII del carcter apuntado.
    movwf    LCD_ValorCaracter        ; Guarda el valor de carcter.
    movf    LCD_ValorCaracter,F        ; Lo nico que hace es posicionar flag 

Z. En caso
    btfsc    STATUS,Z                ; que sea "0x00", que es cdigo 

indicador final   
    goto    LCD_FinMovimiento        ; de mensaje, sale fuera.
LCD_NoUltimoCaracter2
    call    LCD_Caracter            ; Visualiza el carcter ASCII ledo.
    incf    LCD_CursorPosicion,F    ; Contabiliza el incremento de posicin 

del
                                    ; cursor en la pantalla.
    incf    LCD_ApuntaCaracter,F    ; Apunta a la siguiente posicin por 

visualizar.
    goto    LCD_VisualizaCaracter    ; Vuelve a visualizar el siguiente 

carcter
LCD_FinMovimiento                    ; de la lnea.
    return                            ; Vuelve al programa principal.
   
   
